#!/usr/bin/env python
"""
@file    runner.py
@author  Harald Schaefer
@author  Lena Kalleske
@author  Daniel Krajzewicz
@author  Michael Behrisch
@author  Jakob Erdmann
@date    2017-02-07
@version $Id: runner.py 21851 2016-10-31 12:20:12Z behrisch $

Simulation of phase request
based on the
Tutorial for traffic light control via the TraCI interface.

SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
Copyright (C) 2009-2016 DLR/TS, Germany

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import time
import argparse
import subprocess
import random
import Tkinter
try:
  from PIL import ImageTk
except ImportError:
  import ImageTk

sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', "lib"))  # tutorial in tests

import utils

# we need to import python modules from the $SUMO_HOME/tools directory
try:
    sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
    from sumolib import checkBinary
except ImportError:
    sys.exit(
        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")

import traci

parser = argparse.ArgumentParser(description='start SUMO-GUI')
parser.add_argument('-n', '--nogui', action='store_true',
                    default=False, help='run the commandline version of sumo')
parser.add_argument('-v', '--verbose', action='store_true',
                    default=False, help='run the script with more output')
parser.add_argument('-f', '--factor',
                    default=1, help='factor for flows')
#options, args = parser.parse_args()
args = parser.parse_args()

# symbolic names for in/out edges
ausOsten = '-gneE4'
ausSueden = '-gneE25'
ausWesten = '-gneE5.0'
nachSueden = 'gneE25'
nachWesten = 'gneE5.14'
nachOsten = 'gneE4'
pedFromNorth = 'gneE12'
pedToNorth = '-gneE12'
pedFromSouth = '-gneE17'
pedToSouth = 'gneE17'

# symbolic names for colors
blue = '0,0,1'
green = '0,1,0'
red = '1,0,0'
yellow = '1,1,0'
magenta = '1,0,1'
cyan = '1,0,1'

trafficLight = 'junction1'

tl = utils.TrafficLight('junction1', [0, 5, 8], {})
step = 0

def gapFromWest():
    rc = traci.inductionloop.getTimeSinceDetection("w0") > 3
    return rc

def gapFromEast():
    rc = traci.inductionloop.getTimeSinceDetection("e0") > 3 and \
         traci.inductionloop.getTimeSinceDetection("e1") > 3
    return rc

def gapFromSouth():
    rc = traci.inductionloop.getTimeSinceDetection("s0") > 2 and \
         traci.inductionloop.getTimeSinceDetection("s1") > 2
    return rc

def controlJunction1(tl):
    """This function controls the traffic lights for junction1"""
    s = traci.simulation.getCurrentTime() # / 1000
    if (args.verbose):
      print ('{0:8.2f} phase {1:2d} phaseDuration {2:2d}'.format (s/1000., tl.sumoPhase(), tl.remainingPhaseDuration() ) )
      #print (s, "phase", tl.sumoPhase(),
      #        "phaseDuration", tl.remainingPhaseDuration())
    if tl.inSumoPhase(10) and tl.remainingPhaseDuration() == 0 and \
       tl.pedestrianGreenRequested:
      tl.clearPedestrianRequest()
      tl.gotoSumoPhase(13)
    if tl.inSumoPhase(12) and tl.remainingPhaseDuration() == 0:
      tl.gotoSumoPhase(0)
    #if tl.inPhase(1) and tl.phaseRuns() >= 6 and gapFromWest():
    #  tl.terminateCurrentPhase()
    #elif tl.inPhase(2) and tl.phaseRuns() >= 4 and gapFromEast():
    #  tl.terminateCurrentPhase()
    #elif tl.inPhase(3) and tl.phaseRuns() >= 5 and gapFromSouth():
    #  tl.terminateCurrentPhase()

def run(root):
    """execute the TraCI control loop"""
    root.update_idletasks();
    root.update();
    trafficLight = 'junction1'
    tl = TrafficLight('junction1', [0, 5, 8])
    step = 0
    # we start with phase 2 where EW has green
    #traci.trafficlights.setPhase(trafficLight, 2)
    while traci.simulation.getMinExpectedNumber() > 0:
        root.update_idletasks();
        root.update();
        traci.simulationStep()
        controlJunction1(tl)
        step += 1
    traci.close()
    sys.stdout.flush()

def simStep(root):
    """execute one step as callback from the UI"""
    global step
    global tl
    if step == 0:
      tl.setProgram('PhaseRequest')
    traci.simulationStep()
    controlJunction1(tl)
    step += 1
    if traci.simulation.getMinExpectedNumber() <= 0:
      traci.close()
      sys.stdout.flush()
    else:
      root.after_idle(simStep, root)

#def get_options():
#   return options

class StartDialog(Tkinter.Frame):

    def __init__(self, parent):
        Tkinter.Frame.__init__(self, parent)
        # variables for changing language
        self.parent = parent
        self.imageAus = ImageTk.PhotoImage(file="ButtonAus2.jpg")
        self.imageEin = ImageTk.PhotoImage(file="ButtonEin2.jpg")
        self.button = Tkinter.Button(self, image=self.imageAus, command=self.requestGreen)
        self.button.grid(row=0, column=0)
        self.grid()
        # The following three commands are needed so the window pops
        # up on top on Windows...
        self.parent.iconify()
        self.parent.update()
        self.parent.deiconify()

    def requestGreen(self):
        global tl
        tl.requestPedestrianGreenPhase(self)

    def setWaiting(self):
        self.button.configure(image = self.imageEin)

    def clearWaiting(self):
        self.button.configure(image = self.imageAus)

# this is the main entry point of this script
if __name__ == "__main__":
    #options = get_options()

    # this script has been called from the command line. It will start sumo as a
    # server, then connect and run
    if args.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')

    # first, generate the route file for this simulation
    flows = utils.TrafficDef()
    flows.add("westen-osten-",  green, "passenger", ausWesten, nachOsten,   20)
    flows.add("westen-sueden-", red,   "passenger", ausWesten, nachSueden,  20)
    flows.add("osten-westen-",  blue,  "passenger", ausOsten,  nachWesten, 150)
    flows.add("osten-sueden-",  red,   "passenger", ausOsten,  nachSueden,  40)
    flows.add("sueden-osten-",  green, "passenger", ausSueden, nachOsten,   20)
    flows.add("sueden-westen-", blue,  "passenger", ausSueden, nachWesten,  20)
    flows.add("ped-north-south-", blue,  "person", pedFromNorth, pedToSouth,  20, "gneE12 gneE16 gneE17")
    flows.add("ped-south-north-", blue,  "person", pedFromSouth, pedToNorth,  20, "-gneE17 -gneE16 -gneE12")
    flows.generate_routefile('passenger-sorted.flow.xml', float(args.factor))

    root = Tkinter.Tk()
    start = StartDialog(root)
    root.after(100, simStep, root)
    root.after_idle(traci.start, [sumoBinary, "-c", "junction2.sumocfg",
                             "--tripinfo-output", "tripinfo.xml"])
    #


    # this is the normal way of using traci. sumo is started as a
    # subprocess and then the python script connects and runs
    #traci.start([sumoBinary, "-c", "junction.sumocfg",
    #                         "--tripinfo-output", "tripinfo.xml"])
    root.mainloop()
    #run(root)
