#!/usr/bin/env python
"""
@file    runner.py
@author  Harald Schaefer
@author  Lena Kalleske
@author  Daniel Krajzewicz
@author  Michael Behrisch
@author  Jakob Erdmann
@date    2017-02-11
@version $Id: runner.py 21851 2016-10-31 12:20:12Z behrisch $

Simulation of dynamic phases
based on the
Tutorial for traffic light control via the TraCI interface.

SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
Copyright (C) 2009-2016 DLR/TS, Germany

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import time
import argparse
import subprocess
import random

sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', "lib"))

import utils

# we need to import python modules from the $SUMO_HOME/tools directory
try:
    sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
    from sumolib import checkBinary
except ImportError:
    sys.exit(
        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")

import traci

parser = argparse.ArgumentParser(description='start SUMO-GUI')
parser.add_argument('-n', '--nogui', action='store_true',
                    default=False, help='run the commandline version of sumo')
parser.add_argument('-v', '--verbose', action='store_true',
                    default=False, help='run the script with more output')
parser.add_argument('-f', '--factor',
                    default=1, help='factor for flows')
#options, args = parser.parse_args()
args = parser.parse_args()

# symbolic names for in/out edges
ausOsten = '-gneE4'
ausSueden = '-gneE25'
ausWesten = '-gneE5.0'
nachSueden = 'gneE25'
nachWesten = 'gneE5.14'
nachOsten = 'gneE4'
pedFromNorth = 'gneE12'
pedToNorth = '-gneE12'
pedFromSouth = '-gneE17'
pedToSouth = 'gneE17'

# symbolic names for colors
blue = '0,0,1'
green = '0,1,0'
red = '1,0,0'
yellow = '1,1,0'
magenta = '1,0,1'
cyan = '1,0,1'

def controlJunction1(tl):
    """This function controls the traffic lights for junction1"""
    s = traci.simulation.getCurrentTime() # / 1000
    if (args.verbose):
      print (s, "w0", traci.inductionloop.getTimeSinceDetection("w0"),
              "phaseRuns", tl.phaseRuns(),
                     "s1", traci.inductionloop.getTimeSinceDetection("s1"),
                     "ph", traci.trafficlights.getPhase(tl.tlname),
                     "phd", traci.trafficlights.getPhaseDuration(tl.tlname),
                     "nextswitch", traci.trafficlights.getNextSwitch(tl.tlname))
    if tl.inPhase(1) and tl.phaseRuns() >= 6 and tl.gapFrom('West', args.verbose) > 3:
      tl.terminateCurrentPhase()
    elif tl.inPhase(2) and tl.phaseRuns() >= 4 and tl.gapFrom('East', args.verbose) > 3:
      tl.terminateCurrentPhase()
    elif tl.inPhase(3) and tl.phaseRuns() >= 5 and tl.gapFrom('South', args.verbose) > 2:
      tl.terminateCurrentPhase()

def run():
    """execute the TraCI control loop"""
    trafficLight = 'junction1'
    tl = utils.TrafficLight(trafficLight, [0, 5, 8], {'West': ['w0'], 'South': ['s0', 's1'], 'East': ['e0', 'e1']})
    step = 0
    # we start with phase 2 where EW has green
    #traci.trafficlights.setPhase(trafficLight, 2)
    while traci.simulation.getMinExpectedNumber() > 0:
        traci.simulationStep()
        controlJunction1(tl)
        step += 1
    traci.close()
    sys.stdout.flush()


# this is the main entry point of this script
if __name__ == "__main__":
    #options = get_options()

    # this script has been called from the command line. It will start sumo as a
    # server, then connect and run
    if args.nogui:
        sumoBinary = checkBinary('sumo')
    else:
        sumoBinary = checkBinary('sumo-gui')

    # first, generate the route file for this simulation
    flows = utils.TrafficDef()
    flows.add("westen-osten-",  green, "passenger", ausWesten, nachOsten,   20)
    flows.add("westen-sueden-", red,   "passenger", ausWesten, nachSueden,  20)
    flows.add("osten-westen-",  blue,  "passenger", ausOsten,  nachWesten, 150)
    flows.add("osten-sueden-",  red,   "passenger", ausOsten,  nachSueden,  40)
    flows.add("sueden-osten-",  green, "passenger", ausSueden, nachOsten,   20)
    flows.add("sueden-westen-", blue,  "passenger", ausSueden, nachWesten,  20)
    #flows.add("ped-north-south-", blue,  "person", pedFromNorth, pedToSouth,  20, "gneE12 gneE16 gneE17")
    #flows.add("ped-south-north-", blue,  "person", pedFromSouth, pedToNorth,  20, "-gneE17 -gneE16 -gneE12")

    flows.generate_routefile('passenger2.flow.xml', float(args.factor))

    # this is the normal way of using traci. sumo is started as a
    # subprocess and then the python script connects and runs
    traci.start([sumoBinary, "-c", "junction.sumocfg",
                             "--tripinfo-output", "tripinfo.xml"])
    run()
