#!/usr/bin/env python
"""
@file    stream.py
@author  Harald Schaefer
@date    2017-02-11
@version $Id: runner.py 21851 2016-10-31 12:20:12Z behrisch $

Utility for traffic generation

SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
Copyright (C) 2009-2016 DLR/TS, Germany

This file is part of SUMO.
SUMO is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
"""
from __future__ import absolute_import
from __future__ import print_function

import os
import sys
import time
import random

# we need to import python modules from the $SUMO_HOME/tools directory
try:
    sys.path.append(os.path.join(os.path.dirname(
        __file__), '..', '..', '..', '..', "tools"))  # tutorial in tests
    sys.path.append(os.path.join(os.environ.get("SUMO_HOME", os.path.join(
        os.path.dirname(__file__), "..", "..", "..")), "tools"))  # tutorial in docs
    from sumolib import checkBinary
except ImportError:
    sys.exit(
        "please declare environment variable 'SUMO_HOME' as the root directory of your sumo installation (it should contain folders 'bin', 'tools' and 'docs')")

import traci

useLxml = False

try:
  from lxml import etree as ET
  useLxml = True
except ImportError:
  import xml.etree.ElementTree as ET

class TrafficDef:
  """TrafficDef stores traffic definitions and generates a route file for SUMO out of it"""
  def __init__ (self):
    self.flows = []

  def add(self, name, color, type, orig, to, countInIntervall, edges=None):
    self.flows.append(Stream(name, color, type, orig, to, countInIntervall, edges))

  def generate_routefile(self, fileName, factor = 1.0):

    routeNamespace = "http://sumo.dlr.de/xsd/routes_file.xsd"
    namespaceMap =  {None : routeNamespace} # the default namespace (no prefix)

    if useLxml:
      root = ET.Element("routes", nsmap=namespaceMap)
    else:
      root = ET.Element("routes")
    comment1 = ET.Comment(' created with ' + ' '.join(sys.argv) + ' ')
    timeStr = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
    comment2 = ET.Comment(' on           ' + timeStr + ' ')
    comment3 = ET.Comment(' in           ' + os.getcwd() + ' ')
    if useLxml:
      root.addprevious(comment1)
      root.addprevious(comment2)
      root.addprevious(comment3)
    else:
      root.append(comment1)
      root.append(comment2)
      root.append(comment3)
    ET.SubElement(root, "vType", id="passenger",  vClass="passenger")
    ET.SubElement(root, "vType", id="truck",      vClass="truck")
    ET.SubElement(root, "vType", id="trailer",    vClass="trailer")
    ET.SubElement(root, "vType", id="bicycle",    vClass="bicycle")
    ET.SubElement(root, "vType", id="pedestrian", vClass="pedestrian")

    simtime = 0
    timeRange = 10*60
    for i in range(0, 6):
      for stream in self.flows:
        if stream.type != "person":
          stream.createFlow(root, simtime, i, factor)
      for startTime in range(simtime, simtime + timeRange - 1):
        for stream in self.flows:
          if stream.type == "person":
            stream.createPedestrianFlow(root, startTime, timeRange, factor)
      simtime += 10*60

    tree = ET.ElementTree(root)
    if useLxml:
      tree.write(fileName, pretty_print=True, xml_declaration=True, encoding='UTF-8')
    else:
      tree.write(fileName, xml_declaration=True, encoding='UTF-8')
    print("Flows written to {}".format(fileName))

class Stream:
  """
  Stream is an object to create and store different flows of vehicles.
  """
  def __init__ (self, name, color, type, orig, to, countInIntervall, edges=None):
    self.name = name
    self.color = color
    self.type = type
    self.orig = orig
    self.to = to
    self.countInIntervall = countInIntervall
    self.edges = edges

  def createFlow(self, root, time, i, factor):
      val = int(self.countInIntervall * float(factor))
      print(factor, val)
      if val > 0:
        flow = ET.SubElement(root, "flow")
        flow.set("id", self.name + str(i))
        flow.set("color", self.color)
        flow.set("begin", str(time))
        flow.set("end", str(time+10*60))
        flow.set("type", self.type)
        flow.set("from", self.orig)
        flow.set("to", self.to)
        flow.set("number", str(val))

  def createPedestrianFlow(self, parentXml, time, timeRange, factor):
    """Create a pedestrian as child of parentXml"""
    val = int(self.countInIntervall * factor)
    #print('createPedestrianFlow', factor, val)
    probability = (val * 1.0 ) / timeRange;
    randomValue = random.random()
    #print('createPedestrianFlow', randomValue, probability)
    if randomValue <= probability:
        person = ET.SubElement(parentXml, "person")
        person.set('id', self.name + str(time))
        person.set('depart', str(time))
        walk = ET.SubElement(person, "walk")
        walk.set("from", self.orig)
        walk.set("to", self.to)
        walk.set("edges", self.edges)


class TrafficLight:
  """
  The class TrafficLight contains all logic for one traffic light

  Each instance has
  - a name (tlname)
  - a mapping from SUMO phases to logical phases (tls2logic)
  - a mapping from logical phases to SUMO phases (logic2tls)
  """
  def __init__ (self, name, phaseMap, detectorMap):
    self.tlname = name
    self.pedestrianGreenRequested = 0
    self.tls2logic = {}
    self.dir2sensor = detectorMap
    for i,val in enumerate(phaseMap):
      self.tls2logic[val] = i + 1;
    self.logic2tls = phaseMap;
    print(self.logic2tls, self.tls2logic);

  def phaseRuns(self):
    s = traci.simulation.getCurrentTime()
    phd = traci.trafficlights.getPhaseDuration(self.tlname)
    nextSwitch = traci.trafficlights.getNextSwitch(self.tlname)
    return( ( s - (nextSwitch - phd) ) / 1000 )

  def remainingPhaseDuration(self):
    s = traci.simulation.getCurrentTime()
    nextSwitch = traci.trafficlights.getNextSwitch(self.tlname)
    return( ( nextSwitch - s) / 1000 )

  def terminateCurrentPhase(self):
    currentPhase = traci.trafficlights.getPhase(self.tlname)
    currentTime = traci.simulation.getCurrentTime() / 1000
    print ('Finish phase {0:d} after {1:2d} seconds at {2:4d}'.format (self.tls2logic[currentPhase], self.phaseRuns(), currentTime ) )
    # we are not already switching
    traci.trafficlights.setPhase(self.tlname, currentPhase + 1)

  def inPhase(self, logicPhase):
    currentPhase = traci.trafficlights.getPhase(self.tlname)
    if (not(currentPhase in self.tls2logic)):
      return(0)
    rc = self.tls2logic[currentPhase] == logicPhase
    return rc

  def gapFrom(self, dir, verbose=False):
    min = 99999
    for i in self.dir2sensor[dir]:
      val = traci.inductionloop.getTimeSinceDetection(i)
      if (val < min):
        min = val
      if verbose:
        print ('gapFrom {0:5s} = {1:7.3}, min {1:7.3}'.format (i, val, min ) )
    return min

  def inSumoPhase(self, sumoPhase):
    currentPhase = traci.trafficlights.getPhase(self.tlname)
    rc = currentPhase == sumoPhase
    return rc

  def gotoSumoPhase(self, sumoPhase):
    traci.trafficlights.setPhase(self.tlname, sumoPhase)
    print ('Goto phase {}'.format (sumoPhase))

  def setProgram(self, program):
    traci.trafficlights.setProgram(self.tlname, program)

  def sumoPhase(self):
    rc = traci.trafficlights.getPhase(self.tlname)
    return rc

  def requestPedestrianGreenPhase(self, button):
    self.pedestrianGreenRequested = 1
    self.requestButton = button
    self.requestButton.setWaiting()
    print("Green requested")

  def clearPedestrianRequest(self):
    self.pedestrianGreenRequested = 0
    self.requestButton.clearWaiting()

